//
//  BluetoothManager.swift
//  App-RK-BM
//
//  Created by Luis Mora on 28/10/24.
//

import CoreBluetooth

protocol BluetoothManagerDelegate: AnyObject {
    func didUpdateConnectionStatus(isConnected: Bool)
    func didReceiveData(fromCharacteristic uuid: String, data: String)
}

class BluetoothManager: NSObject, CBCentralManagerDelegate, CBPeripheralDelegate {
    static let shared = BluetoothManager() // Singleton para un acceso fácil
    
    // Define los UUIDs
    private let SERVICE_UUID = CBUUID(string: "4fafc201-1fb5-459e-8fcc-c5c9c331914b")
    private let CHARACTERISTIC_UUID_CMD = CBUUID(string: "beb5483e-36e1-4688-b7f5-ea07361b26a8")
    private let CHARACTERISTIC_UUID_MAX = CBUUID(string: "beb5483e-36e1-4688-b7f5-ea07361b26b1") // UUID ficticio para MAX3010
    private let CHARACTERISTIC_UUID_MPU = CBUUID(string: "beb5483e-36e1-4688-b7f5-ea07361b26b2") // UUID ficticio para MPU6050
    
    private var centralManager: CBCentralManager!
    private var esp32Peripheral: CBPeripheral?
    private var characteristics: [CBUUID: CBCharacteristic] = [:] // variable para almacenar características encontradas
    
    private let deviceName = "ESP_32_BLE_ITM" // Nombre del wearable para realizar conexión
    
    weak var delegate: BluetoothManagerDelegate? // delegado para notificaciones
    
    override init () {
        super.init()
        self.centralManager = CBCentralManager(delegate: self, queue: nil)
    }
    
    // inicia el escaneo de dispositivos
    func startScanning() {
        if centralManager.state == .poweredOn {
            // centralManager.scanForPeripherals(withServices: [SERVICE_UUID], options: nil)
            centralManager.scanForPeripherals(withServices: nil, options: nil)
            print("Buscando ESP32...")
        }
    }
    
    // detiene la conexión manualmente
    func disconnect() {
        if let esp32Peripheral = esp32Peripheral {
            centralManager.cancelPeripheralConnection(esp32Peripheral)
            print("Desconectando el ESP32...")
        }
    }
    
    // CBCentralManagerDelegate - Estado de Bluetooth
    func centralManagerDidUpdateState(_ central: CBCentralManager) {
        if central.state != .poweredOn {
            print("BT no esta disponible")
            delegate?.didUpdateConnectionStatus(isConnected: false)
        }
    }
    
    // CBCentralManagerDelegate - Detectar y conectar al ESP32
    func centralManager(_ central: CBCentralManager, didDiscover peripheral: CBPeripheral, advertisementData: [String: Any], rssi RSSI: NSNumber) {
        let deviceName = peripheral.name ?? "Sin nombre"
        let uuid = peripheral.identifier.uuidString
        print("Dispositivo detectado: \(deviceName) - UUID: \(uuid)")
        
        if peripheral.name == self.deviceName {
            self.esp32Peripheral = peripheral
            self.esp32Peripheral?.delegate = self
            centralManager.stopScan()
            centralManager.connect(esp32Peripheral!, options: nil)
            print("Conectando a \(peripheral.name ?? "ESP32")")
        }
    }
    
    // CBCentralManagerDelegate - Conexión exitosa
    func centralManager(_ central: CBCentralManager, didConnect peripheral: CBPeripheral) {
        print("Dispositivo iOS conectado a: \(self.deviceName)")
        delegate?.didUpdateConnectionStatus(isConnected: true)
        esp32Peripheral?.discoverServices(nil) // descubrir todos los servicios
    }
    
    // CBCentralManagerDelegate - Desconexión
    func centralManager(_ central: CBCentralManager, didDisconnectPeripheral peripheral: CBPeripheral, error: Error?) {
        if peripheral == esp32Peripheral {
            print("Se ha desconectado el ESP32")
            delegate?.didUpdateConnectionStatus(isConnected: false)
        }
    }
    
    // Descubrir servicios en el ESP32
    func peripheral(_ peripheral: CBPeripheral, didDiscoverServices error: Error?) {
        guard let services = peripheral.services else { return }
        for service in peripheral.services ?? [] {
            print("Servicio descubierto: \(service.uuid.uuidString)")
            peripheral.discoverCharacteristics(nil, for: service) // Descubrir todas las características
        }
    }
        
    // Descubrir caracteristicas
    func peripheral(_ peripheral: CBPeripheral, didDiscoverCharacteristicsFor service: CBService, error: Error?) {
        guard let characteristics = service.characteristics else { return }
        for characteristic in characteristics {
            print("Característica encontrada: \(characteristic.uuid.uuidString)")
            if characteristic.properties.contains(.notify) {
                peripheral.setNotifyValue(true, for: characteristic)
            }
            self.characteristics[characteristic.uuid] = characteristic // guardar las caracteristicas
        }
    }
        
    // Leer datos recibidos
    func peripheral(_ peripheral: CBPeripheral, didUpdateValueFor characteristic: CBCharacteristic, error: Error?) {
        guard let data = characteristic.value, let receivedString = String(data: data, encoding: .utf8) else { return }
        print("Datos recibidos de \(characteristic.uuid): \(receivedString)")
        delegate?.didReceiveData(fromCharacteristic: characteristic.uuid.uuidString, data: receivedString)
    }
        
    // Enviar un comando al ESP32
    func sendCommand(_ command: String) {
        guard let cmdCharacteristic = characteristics[CHARACTERISTIC_UUID_CMD] else { return }
        let data = Data(("!" + command + "$").utf8)
        esp32Peripheral?.writeValue(data, for: cmdCharacteristic, type: .withResponse)
        print("Comando enviado: \(command)")
    }
    
    // Función para buscar dispositivos BLE / depuraciones
    func searchForDevices() {
        if centralManager.state == .poweredOn {
            centralManager.scanForPeripherals(withServices: nil, options: nil)
            print("Buscando dispositivos BLE...")
            DispatchQueue.main.asyncAfter(deadline: .now() + 2) {
                self.centralManager.stopScan()
                print("Deteniendo escaneo")
            }
        } else {
            print("Bluetooth no esta disponible")
        }
    }
}
